//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the op1a file interface.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#ifndef OC_I_OP1A_FILE_HPP
#define OC_I_OP1A_FILE_HPP

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_mxf_file.hpp>

namespace opencube
{
  namespace MXFTK_NAMESPACE
  {
    /* I_op1a_file:
     * An interface to create an op1a mxf file.
     */
    class I_op1a_file : public I_mxf_file
    {
    private:
      virtual size_t CALLING_CONVENTION write(const uint8_t*, size_t)=0;
      virtual bool CALLING_CONVENTION end_of_stream()=0;
    public:
      /*
       * Set the material to be wrapped in an mxf_file.
       * Output generic material will also be automatically created upon calling of this function.
       * Returns true if successful.
       */
      virtual bool CALLING_CONVENTION set_material(I_concrete_material* /* in */)=0;

      /*
       * Set the material to be included as an external reference in this mxf file.
       * Output material will also be automatically created upon calling of this function.
       * the char* is used to set the locator value of the material descriptor.
       * This is usually the path to the MXF file containing the external reference.
       * The second parameter is used to set the locator string (usually the path
       * to the external file) that will be included in the header metadata.
       * Returns true if successful.
       */
      virtual bool CALLING_CONVENTION set_external_material(I_concrete_material* /* in */, 
                                                            const wchar_t* /* in free */)=0;
    
      /*
       * Return the generic material created after calling set_material or set_external_material
       * Return NULL if the generic material was not created yet.
       */
      virtual I_generic_material* CALLING_CONVENTION get_current_generic_material()=0;
    };

    /*
     * Retrieves an I_op1a_file interface.
     * I_op1a_file** pointer to the newly allocated object
     * const char* path to the mxf file to be written.
     * Call to flush() will effectively cause the on-disk writing of
     * the file.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewOp1aFileInterface(I_op1a_file** /* out free */, 
                                                                       const char* /* in free */);

    /*
     * Retrieves an I_op1a_file interface.
     * I_op1a_file** pointer to the newly allocated object
     * OpZero is a constraint to Op1a. The concrete material you will add to
     * this file cannot contain more than one essence file and you won't be 
     * able to change the parameters of the file.
     * const char* path to the mxf file to be written.
     * Call to flush() will effectively cause the on-disk writing of
     * the file.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewOpZeroFileInterface(I_op1a_file** /* out free */, 
                                                                         const char* /* in free */);

    /*
     * Retrieves an I_op1a_file interface whose flush will be performed in streaming mode.
     * I_op1a_file** pointer to the newly allocated object
     * const char* path to the mxf file to be written.
     * Call to flush() will effectively cause the on-disk writing of
     * the file.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewOp1aStreamInterface(I_op1a_file** /* out free */, 
                                                                         I_output_mxf_stream_task* /* in free */);

    /*
     * Retrieves an I_op1a_file interface.
     * Retrieves an I_op1a_file interface whose flush will be performed in streaming mode.
     * OpZero is a constraint to Op1a. The concrete material you will add to
     * this file cannot contain more than one essence file and you won't be 
     * able to change the parameters of the file.
     * The streaming interface you provide MUST implement the interfaces in order to seek
     * in the output MXF stream. 
     * const char* path to the mxf file to be written.
     * Call to flush() will effectively cause the on-disk writing of the file.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewOpZeroStreamInterface(I_op1a_file** /* out free */, 
                                                                           I_output_mxf_stream_task* /* in free */);

    /*
     * Frees an op1a_file interface.
     * I_op1a_file** pointer to the object to be freed.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeOp1aFileInterface(I_op1a_file** /* in out */);
  }
}

#endif
