//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the op2c file interface.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#ifndef OC_I_OP2C_FILE_HPP
#define OC_I_OP2C_FILE_HPP

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_mxf_file.hpp>

namespace opencube
{
  namespace MXFTK_NAMESPACE
  {
    /* I_op2c_file:
     * An interface to create an op1c mxf file.
     */
    class I_op2c_file : public I_mxf_file
    {
    private:
      virtual size_t CALLING_CONVENTION write(const uint8_t*, size_t)=0;
      virtual bool CALLING_CONVENTION end_of_stream()=0;
    public:
      /*
       * This function creates a new generic material (Material Package)
       * The following calls to new_multiplexed_material(), add_material_track() and add_external_material_track()
       * will apply to the newly created generic material.
       */
      virtual bool CALLING_CONVENTION new_generic_material()=0;

      /*
       * Notifies MXFTk that the next tracks added to this file will be multiplexed
       * For instance to create the following generic material (a,b,c,d,e,f designate 
       * concrete tracks) :
       * track 1 : a+b+c
       * track 2 : d+e+f
       * you should do the following calls:
       * new_generic_material()
       * new_multiplexed_track()
       * add_material_track(conc_mat, a)
       * add_material_track(conc_mat, d)
       * new_multiplexed_track()
       * add_material_track(conc_mat, b)
       * add_material_track(conc_mat, e)
       * new_multiplexed_track()
       * add_material_track(conc_mat, c)
       * add_material_track(conc_mat, f)
       *
       * Note that the concrete tracks don't need to come from the same concrete material.
       * However, each set of multiplexed tracks should have the same duration.
       */
      virtual bool CALLING_CONVENTION new_multiplexed_track()	=0;

      /*
       * Add to the current generic material the data referenced by the given 
       * concrete_track from the given concrete_material
       * All the concrete tracks added to the same generic material will be played
       * parallely and should have the same duration.
       */
      virtual bool CALLING_CONVENTION add_material_track(I_concrete_material* /* in */, 
                                                         I_concrete_track* /* in */)=0;

      /*
       * Add to the current generic material the data referenced by the given 
       * concrete_track from the given concrete_material.
       * The concrete material will remain external to the file.
       * All the concrete tracks added to the same generic material will be played
       * parallely and should have the same duration.
       * The second parameter is used to set the locator string (usually the path
       * to the external file) that will be included in the header metadata.
       */
      virtual bool CALLING_CONVENTION add_external_material_track(I_concrete_material* /* in */, 
                                                                  I_concrete_track* /* in */, 
                                                                  const wchar_t* /* in free */)=0;

      /*
       * Call this function to state whether special processing is required at the join of two 
       * source clips of a track of the generic material. This value is used by the MXF decoder
       * to know if it can play continuously the essences. 
       * For instance if the file will contain a DV and MPEG in the same track, you should
       * set this value to false (default). Whenever special processing is required to
       * play at least one of the generic material's track, this value should be set to false.
       * Even if only a single track from a single material does not allow continuous decoding
       * the boolean should be set to false.
      */
      virtual bool CALLING_CONVENTION set_continuous_decoding(bool)=0;

      /*
       * Return the generic material created after calling add_material or add_external_material
       * Return NULL if you just called new_generic_material and did not call add_material
       * or add_new_material
       * Do not call add_material or add_external_material after calling this function
       * or all the changes you made to the current generic material will be lost.
       */
      virtual I_generic_material* CALLING_CONVENTION get_current_generic_material()=0;
    };

    /*
     * Retrieves an I_op2c_file interface.
     * I_op2c_file** pointer to the newly allocated object
     * const char* path to the mxf file to be written.
     * Call to flush() will effectively cause the on-disk writing of
     * the file.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewOp2cFileInterface(I_op2c_file** /* out free */, 
                                                                       const char* /* in free */);

    /*
     * Retrieves an I_op2c_file interface whose flush will be performed in streaming mode.
     * I_opc_file** pointer to the newly allocated object
     * const char* path to the mxf file to be written.
     * Call to flush() will effectively cause the on-disk writing of
     * the file.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewOp2cStreamInterface(I_op2c_file** /* out free */, 
                                                                         I_output_mxf_stream_task* /* in free */);

    /*
     * Frees an op2c_file interface.
     * I_op2c_file** pointer to the object to be freed.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeOp2cFileInterface(I_op2c_file** /* in out */);
  }
}

#endif
