//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the property interface.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#ifndef OC_I_PROPERTY_HPP
#define OC_I_PROPERTY_HPP

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_value.hpp>

namespace opencube
{
  namespace MXFTK_NAMESPACE
  {
    /* label_c:
     * A const char* indicating the property or metadata name.
     * Property name should strictly follow the naming convention
     * of the dictionary. For instance "DMS1:ProductionFramework"
     * or "file_format:CompanyName"
     */
    typedef const char* label_c;

    /* I_value:
     * This class represents a property from a metadata tree.
     * Is has a label and a value. The value can be a reference
     * to a metadata enabling to create an arborescence similar 
     * to the MXF data structure.
     */
    class I_property
    {
    public:
      /**
       * Returns the label. "" if an error occured. 
       */
      virtual label_c CALLING_CONVENTION get_label() const=0;

      /**
       * Returns the value.
       * Returns NULL if an error occured.
       */
      virtual const I_value* CALLING_CONVENTION get_value() const=0;

      /**
       * Returns a copy of the all the metadata sets allowed for this property
       * according to the dictionary.
       * List is empty if property is not a set.
       * This data MUST be deleted using free_metadata_sets()
       * Returns NULL if an error occured.
       */
      virtual metadata_list* /* free */ CALLING_CONVENTION get_dic_metadata_sets() const=0;

      /**
       * Frees metadata list returned by get_dic_metatada_sets.
       * Returns true if successful
       */
      virtual bool CALLING_CONVENTION free_metadata_sets(metadata_list* /* in */) const=0;
    };

    /*
     * Retrieves a property interface.
     * I_property** pointer to the value interface to be created.
     * label_c name of the property.
     * I_value* value of the property.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewPropertyInterface(I_property** /* out free */, 
                                                                       label_c /* in free */, 
                                                                       const I_value* /* in free */);
	
    /*
     * Frees a property interface. The I_value* interface used to create this
     * property in NewPropertyInterface won't be deleted. It must be freed
     * using FreeValueInterface.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreePropertyInterface(I_property** /* in out */);
  }
}

#endif
