//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the value interface.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#ifndef OC_I_VALUE_HPP
#define OC_I_VALUE_HPP

#include <mxf_tk/mxf_tk.hpp>

namespace opencube
{
  namespace MXFTK_NAMESPACE
  {
    /* value_type:
     * MXF_TK types followed by their format scheme
     * Note: 64-byte umid is currently not implemented
     * Note: MXF_STRONG_REF, MXF_WEAK_REF, MXF_ARRAY_STRONG_REF
     * and MXF_ARRAY_WEAK_REF are pointers to I_metadata 
     * structure. This is used to create an arborescence enabling
     * to organize metadata in a tree structure.
     */
    typedef enum {
      MXF_BOOLEAN=uint32_t(0)	,	// int8_t
      MXF_INT8		        ,	// int8_t
      MXF_INT16		        ,	// int16_t
      MXF_INT32			,	// int32_t
      MXF_INT64			,	// int64_t
      MXF_UINT8			,	// uint8_t
      MXF_UINT16		,	// uint16_t
      MXF_UINT32		,	// uint32_t
      MXF_UINT64		,	// uint64_t
      MXF_UUID			,	// uint8_t[16] (also uses this type for UL values)
      MXF_UMID32	        ,	// I_umid*
      MXF_UMID64		,	// I_umid64*
      MXF_RATIONAL		,	// uint32_t[2] // num / den
      MXF_TIMESTAMP		,	// uint16_t[7] // year / month / day / hour / minute / second / millisecond 
      MXF_ISO7_STRING		,	// char[n]
      MXF_UTF16_STRING		,	// wchar_t[n]
      MXF_PRODUCT_VERSION	,	// uint16_t[5] // major version / minor version / patch / build / release
      MXF_STRONG_REF		,	// I_metadata*
      MXF_WEAK_REF		,	// I_metadata*
      MXF_ARRAY_BOOLEAN		,	// int8_t[n]
      MXF_ARRAY_INT8		,	// int8_t[n]
      MXF_ARRAY_INT16		,	// int16_t[n]
      MXF_ARRAY_INT32		,	// int32_t[n]
      MXF_ARRAY_INT64		,	// int64_t[n]	
      MXF_ARRAY_UINT8		,	// uint8_t[n]
      MXF_ARRAY_UINT16		,	// uint16_t[n]
      MXF_ARRAY_UINT32		,	// uint32_t[n]
      MXF_ARRAY_UINT64		,	// uint64_t[n]
      MXF_ARRAY_UUID		,	// uint8_t[16*n] 
      MXF_ARRAY_STRONG_REF	,	// I_metadata*[n]
      MXF_ARRAY_WEAK_REF	,	// I_metadata*[n]
      MXF_ARRAY_UMID32		,	// I_umid*[n]
      MXF_ARRAY_UMID64		,	// I_umid64*[n]   
      MXF_UNKNOWN	
    } value_type;

    /**
     * These two functions are provided to easily commute from utf16 string to iso7 string 
     * data types (however converting from utf16 to iso7 may result in data loss).
     * The last int parameter indictaes the number of characters in the string 
     * (i.e not the length in bytes!)
     * Returns true if successful.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION iso7_to_utf16(wchar_t* /* out free */, 
                                                                const char* /* in free */, 
                                                                int /* in */);
    extern "C" mxf_tk_API bool CALLING_CONVENTION utf16_to_iso7(char* /* out free */, 
                                                                const wchar_t* /* in free */, 
                                                                int /* in */);
	
    /* I_value:
     * This class is used to represent a value to be set with a property.
     */
    class I_value
    {
    public:
      /**
       * Returns type of this value.
       */
      virtual value_type CALLING_CONVENTION get_type() const=0;

      /**
       * Returns size of this value (in bytes).
       */
      virtual size_t CALLING_CONVENTION get_size() const=0;

      /**
       * Returns data (must be casted to the appropriate type)
       * Returned data must not be edited nor deleted.
       */
      virtual const uint8_t* CALLING_CONVENTION get_data() const=0;
    };

    /*
     * Retrieves a value interface.
     * I_value** pointer to the value interface to be created.
     * value_type type of the value to be created.
     * size_t size of the value (in bytes)
     * uint8_t* buffer containing the data.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewValueInterface(I_value** /* out free */, 
                                                                    value_type /* in */, 
                                                                    size_t /* in */, 
                                                                    const uint8_t* /* in free */);

    /*
     * Frees a value interface. The uint8_t* data buffer used to create this
     * interface won't be deleted. It is the MXF_TK user responsibility.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeValueInterface(I_value** /* in out */);
  }
}

#endif
