//------------------------------------------------------------------------------
//  Author:    	Thierry FRANZETTI
//  Comment:	MXF_TK: An example to generate an op1a mxf file and set
//  its timecode.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#if defined(BC6)	// Borland C Builder 6.0
#include <vcl.h>
#endif

#include <mxf_tk/I_concrete_material.hpp>
#include <mxf_tk/I_timecode_material.hpp>
#include <mxf_tk/I_op1a_file.hpp>

#include <iostream>
#include <sstream>

using namespace opencube::MXFTK_NAMESPACE;
using namespace std;

// MxfTk error handler
I_mxf_error_handler* err_h = NULL;

//Throws an exception when an error occured 
void throw_on_error(); 

//Called if an error occured.
void display_errors();

//Timecode redefinition
void redefine_timecode(I_op1a_file*);

// To display the timecode generated by MxfTk
void read_timecode(I_generic_material*);

int main(int argc, char* argv[])
{    
  if (argc < 4)
  {
    wcout << L"usage : out.mxf out.xml source file [...source file]" << endl;
    exit(0);
  }

  // the op1a mxf file we are going to write.
  I_op1a_file* pi_op1a_file = NULL;
  // the interface to manipulate the source to be wrapped.
  I_concrete_material* pi_conc_mat = NULL;
  // this boolean is set to true if pi_conc_mat needs to be destroyed
  bool delete_conc_mat = false;
  // Path to the source files to be wrapped
  locators* pi_loc = NULL;

  try
  {
    wcout << L"Retrieve MxfTk Error Handler" << endl;
    if (!GetMxfErrorHandlerInterface(&err_h))
      throw 1;

    wcout << L"MxfTk Initialization" << endl;
    InitMXF_TK();
    throw_on_error();

    wcout << L"Create the op1a mxf file interface" << endl;
    NewOp1aFileInterface(&pi_op1a_file, argv[1]);
    throw_on_error();

    wcout << L"Build source files list" << endl;
    GetEmptyLocatorInterface(&pi_loc);
    throw_on_error();
    for (int i=3; i<argc; i++)
      pi_loc->add((const char**)&(argv[i]));

    wcout << L"Create source material" << endl;
    NewConcreteMaterialInterface(&pi_conc_mat, pi_loc, frame_wrapping);
    throw_on_error();
    delete_conc_mat = true;

    wcout << L"Attach material to the file" << endl;
    pi_op1a_file->set_material(pi_conc_mat); // pi_conc_mat will be deleted upon destruction of pi_op1a_file
    throw_on_error();
    delete_conc_mat = false;

    redefine_timecode(pi_op1a_file);
    throw_on_error();

    wcout << L"Write on disk" << endl;
    pi_op1a_file->flush();
    throw_on_error();

    wcout << L"Wait termination" << endl;
    pi_op1a_file->wait_end_flush_thread();
    throw_on_error();

    wcout << L"Output header metadata XML file" << endl; // optional!
    pi_op1a_file->output_xml(argv[2]);
    throw_on_error();
  }
  catch (int)
  {
    wcout << L"An error occured!" << endl;
    // Retrieving details of the error(s) and display it(them).
    display_errors();
  }

  wcout << L"Close Process" << endl;
  if (pi_op1a_file)
    FreeOp1aFileInterface(&pi_op1a_file);
  if (pi_conc_mat && delete_conc_mat)
    FreeConcreteMaterialInterface(&pi_conc_mat);
  if (pi_loc)
    FreeLocatorInterface(&pi_loc);

  return 0;
}

void redefine_timecode(I_op1a_file* pi_op1a_file)
{
  // Output material
  I_generic_material* pi_gen_mat = NULL;

  // New starting timecode value
  I_timecode* pi_new_timecode = NULL;
  bool delete_new_timecode = false;

  // New timecode component (to add to timecode material) 
  I_timecode_component* pi_time_comp = NULL;

  // New timecode material
  I_timecode_material* pi_new_time_mat = NULL;
  bool delete_time_mat = false;

  try
  {
    wcout << L"\tGet output material (new timecode will be added to this material)" << endl;
    generic_list* gen_list = pi_op1a_file->output_material();
    gen_list->begin();
    pi_gen_mat = *gen_list->next();
    pi_op1a_file->free_generic_list(gen_list);
    throw_on_error();

    wcout << L"\tCurrent Timecode is..." << endl;
    read_timecode(pi_gen_mat);
    throw_on_error();

    wcout << L"\tRetrieve output timecode that was automatically generated by MxfTk" << endl;
    I_timecode_material* pi_time_mat = pi_gen_mat->get_timecode();
    int64_t origin = pi_time_mat->get_origin();
    int64_t duration = pi_time_mat->get_total_duration();
    rational *edit_rate = pi_time_mat->get_edit_rate();
    throw_on_error();

    wcout << L"\tTimecode Redefinition" << endl;
    wcout << L"\tBuild new timecode material" << endl;
    NewTimecodeMaterialInterface(&pi_new_time_mat, origin, edit_rate);
    throw_on_error();
    delete_time_mat = true;

    wcout << L"\tSet new start timecode value" << endl;
    NewTimecodeInterface(&pi_new_timecode, "00:05:00:00", uint16_t(edit_rate->get_double()+0.5) /* closest integer*/, false); 
    FreeRationalInterface(&edit_rate);
    throw_on_error();
    delete_new_timecode = true;

    wcout << L"\tCreate new timecode component and append it to timecode material" << endl; 
    NewTimecodeComponentInterface(&pi_time_comp, pi_new_timecode, duration);
    delete_new_timecode = false; //pi_new_timecode will be deleted upon destruction of pi_time_comp
    pi_new_time_mat->append_timecode_component(pi_time_comp);
    throw_on_error();

    wcout << L"\tAttach New Timecode material to output material" << endl;
    pi_gen_mat->set_timecode(pi_new_time_mat); 
    throw_on_error();
    delete_time_mat = false; // pi_time_mat will be deleted upon destruction of pi_gen_mat

    wcout << L"\tCurrent Timecode is..." << endl;
    read_timecode(pi_gen_mat);
  }
  catch(int)
  {
    // do nothing will be rethrown in the following throw_on_error() 
    // and catched in main()
  }

  if (delete_time_mat)
    FreeTimecodeMaterialInterface(&pi_new_time_mat);

  if (delete_new_timecode)
    FreeTimecodeInterface(&pi_new_timecode);

  throw_on_error();
}

void read_timecode(I_generic_material* pi_gen_mat)
{
  // Output timecode material
  I_timecode_material* pi_time_mat = NULL;

  // List of timecode components fro mthe timecode material
  ordered_timecode_list* time_list = NULL;

  try
  {
    pi_time_mat = pi_gen_mat->get_timecode();

    wstringstream str;
    rational *edit_rate = pi_time_mat->get_edit_rate();
    str << L"\t\tTimecode track: origin:" << (int)pi_time_mat->get_origin() << L"  duration:"  << (int)pi_time_mat->get_total_duration() << L"  edit rate:" << (int)edit_rate->get_num() << L"/" << (int)edit_rate->get_den() << endl;
    wcout << str.str(); 
    FreeRationalInterface(&edit_rate);
    throw_on_error();

    wcout << L"\t\tTimecode: " << endl;
    time_list = pi_time_mat->get_timecode_list();
    time_list->begin();
    I_timecode_component** tmc_p = NULL;
    while ((tmc_p = time_list->next())!=NULL)
    {
      I_timecode_component* tmc = *tmc_p;
      I_timecode* pi_tm = NULL;
      CopyTimecodeInterface(&pi_tm, tmc->get_timecode());	// copy it because we will change its value
      int64_t l_dur = tmc->get_duration();
      wcout << L"\t\t\t" << pi_tm->get_time() << L" until ";
      if (l_dur>=0)
      {
        pi_tm->add(l_dur-1);
        wcout << pi_tm->get_time() << endl;
      }
      else
      {
        wcout << L"??" << endl;
      }
      FreeTimecodeInterface(&pi_tm);
      throw_on_error();
    }  
  }
  catch(int)
  {
    // do nothing will be rethrown in the following throw_on_error() 
    // and catched in redefine_timecode()
  }

  if (time_list)
    pi_time_mat->free_timecode_list(time_list);

  throw_on_error();
}

void throw_on_error()
{
  if (err_h->has_an_error())
    throw 1;
}

void display_errors()
{
  // Retrieve error handler
  if (!err_h)
    return;

  // Retrieve list of errors
  error_list* err_l = err_h->get_errors();
  if (err_l)
  {
    err_l->begin();
    I_mxf_error** ppi_mxf_error = NULL;
    while ((ppi_mxf_error = err_l->next()) != NULL) //go through the list
    {
      I_mxf_error* pi_error = *ppi_mxf_error;
      // and display each error 
      wcout << L"Error " << (int)pi_error->get_id() << L": " << pi_error->get_error_msg() << endl;
    }
    // free error list interface
    err_h->free_errors(err_l);
  }
}
