//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the metadata interface.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#ifndef OC_I_METADATA_HPP
#define OC_I_METADATA_HPP

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_property.hpp>

namespace opencube
{
  namespace MXFTK_NAMESPACE
  {
    /* I_metadata:
     * This class allows the manipulation of descriptive metadata
     * arranged in a tree structure. Each metadata object contains
     * a set of properties. Each property associate a label with a value.
     * This value can be a metadata, therefore using this mechanism
     * it becomes possible to build a metadata tree starting from a metadata
     * root. This root can be used to create an I_metadata_material that will
     * be appended to a metadata I_track.
     * See example2 and example4 for details of implementation.
     */
    class I_metadata
    {
    public:
      /**
       * Adds a property to the current metadata. The content of the property will
       * be copied. Therefore, it is possible to free the I_property* interface
       * just after adding it.
       * If the metadata already contains a property with the same label, it will
       * be deleted and replaced by the new one. The only exception occurs when
       * the property is of type MXF_ARRAY_STRONG_REF or MXF_ARRAY_WEAK_REF, in that case
       * referenced I_metadata* interfaces will be added to the ones already stored
       * in the internal metadata property. This allows to add "branches" to the current
       * tree without deleting the ones previously created.
       * Returns true if successful.
       */
      virtual bool CALLING_CONVENTION add(const I_property* /* in free */)=0;

      /**
       * Removes a property from the metadata. If the property references
       * other metadata branches from the tree; they will also be deleted.
       * Returns true if successful.
       */
      virtual bool CALLING_CONVENTION remove(const I_property* /* in free */)=0;

      /**
       * Returns a copy of the properties of this metadata.
       * This data MUST be deleted using free_properties().
       * Returns NULL if an error occured.
       */
      virtual properties_list* /* free */ CALLING_CONVENTION get_properties() const=0;

      /**
       * Returns a copy of the all the properties allowed for this metadata
       * according to the dictionary.
       * This data MUST be deleted using free_properties()
       * Returns NULL if an error occured.
       */
      virtual properties_list* /* free */ CALLING_CONVENTION get_dic_properties() const=0;

      /**
       * Frees properties returned by get_properties and get_dic_properties.
       * Returns true if successful
       */
      virtual bool CALLING_CONVENTION free_properties(properties_list* /* in */) const=0;

      /**
       * Gets label of the current metadata.
       * Returns an empty label ("") if an error occured.
       */
      virtual label_c CALLING_CONVENTION get_label() const=0;

      /**
       * Outputs the current metadata tree in an xml file
       * Returns true if successful.
       */
      virtual bool CALLING_CONVENTION output_xml(const char* /* in free */)=0;
    };

    /*
     * Retrieves a metadata interface.
     * I_metadata** pointer to the metadata interface to be created.
     * label name of the metadata to be created (for instance "DMS1:ProductionFramework").
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMetadataInterfaceByLabel(I_metadata** /* out free */, 
                                                                              label_c /* in free */);
 
    /*
     * Builds a metadata interface from an XML file.
     * I_metadata** pointer to the metadata interface to be created.
     * const char* path to the XML file describing the metadata tree.
     * The XML file should effectively describes a tree structure (i.e one and
     * only one of the metadata described by the file should be a root; no cycles allowed). 
     * Failure to respect this requirement will throw an error or may result 
     * in an hectic behavior. 
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewMetadataInterfaceByXML(I_metadata** /* out free */, 
                                                                            const char* /* in free */);
 
    /**
     * Frees metadata interface. Brnaches referenced by this metadata
     * will also be deleted. Therfore, freeing the root metadata will
     * delete the entire tree.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeMetadataInterface(I_metadata** /* in out */);

    /* 
     * This macro can be used to create a new property
     * in an existing I_metadata interface.
     */
#define ADD_PROPERTY(meta /* I_metadata* */,         \
                     name /* label_c */,             \
                     type /* value_type */,          \
                     size /* size_t */,              \
                     data /* pointer to the data */) \
    {                                                \
      I_value* pi_val = NULL;                                       \
      I_property* pi_prop = NULL;                                   \
      NewValueInterface(&pi_val, type, size, (const uint8_t*)data); \
      NewPropertyInterface(&pi_prop, name, pi_val);                 \
      meta->add(pi_prop);                                           \
      FreeValueInterface(&pi_val);                                  \
      FreePropertyInterface(&pi_prop);                              \
    } /* pointer data can be freed after calling this macro */
  }
}

#endif
