//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the timecode material interface.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#ifndef OC_I_TIMECODE_MATERIAL_HPP
#define OC_I_TIMECODE_MATERIAL_HPP

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_generic_material.hpp>
#include <mxf_tk/I_timecode.hpp>


namespace opencube
{	
  namespace MXFTK_NAMESPACE
  {
    /*
     * A timecode component defines a timecode and a duration (-1 if unknown)
     * Duration is expressed in edit units of the timecode track to which
     * this component will be added.
     * It is the indivisible item of a timecode track (timecode material).
     */
    class I_timecode_component
    {
    public:
      virtual I_timecode* CALLING_CONVENTION get_timecode()=0;
      virtual int64_t CALLING_CONVENTION get_duration()=0;
    };

    /*
     * Retrieves a timecode component interface.
     * I_timecode_component** pointer to the newly allocated object
     * I_timecode* starting timecode
     * int64_t duration (in edit units of the timecode material)
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewTimecodeComponentInterface(I_timecode_component** /* out free */, 
                                                                                I_timecode* /* in */, 
                                                                                int64_t /* in */);

    /*
     * Free a timecode component interface.
     * I_timecode_component** pointer to the object to be freed
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeTimecodeComponentInterface(I_timecode_component**  /* in out */);
      
    /*
     * A timecode material is a track describing the file playout (in a
     * generic material) and the file "play-in" (in a concrete material).
     * It is defined by a set of timecode components. Each timecode component
     * holds a starting timecode value and a duration. Discontinuous timecodes
     * can be set by redefining the timecode during the file play-in, this can
     * be achieved by appending several timecode components to the material.
     * (See example3 for details of implementation).
     */
    class I_timecode_material : public I_generic_material
    {
    private:
      /*
       * Disabled I_timecode_material functions.
       */
      virtual I_track* CALLING_CONVENTION new_metadata_track(track_type, int64_t, rational*)=0;
      virtual I_track* CALLING_CONVENTION new_stream_metadata_track(I_input_metadata_stream_task*, track_type, int64_t, rational*)=0;
      virtual track_list* CALLING_CONVENTION source_tracks()=0;
      virtual track_list* CALLING_CONVENTION metadata_tracks()=0;
      virtual bool CALLING_CONVENTION free_tracks(track_list*)=0;
      virtual I_timecode_material* CALLING_CONVENTION get_timecode() const=0;
      virtual bool CALLING_CONVENTION set_timecode(I_timecode_material*)=0;
      virtual bool CALLING_CONVENTION set_umid(const I_umid*)=0;
      virtual I_umid* CALLING_CONVENTION get_umid()=0;
    public:
      /*
       * Returns the edit rate of this timecode material.
       * Returns NULL if an error occured
       */
      virtual rational* /* free */ CALLING_CONVENTION get_edit_rate()=0;

      /*
       * Returns the total duration of this timecode material
       * Returns -1 if unknown
       */
      virtual int64_t CALLING_CONVENTION get_total_duration() const=0;

      /*
       * Returns the origin of this timecode material
       */
      virtual int64_t CALLING_CONVENTION get_origin() const=0;

      /*
       * To create a continuous timecode a single timecode component
       * with the total playin/playout duration should be appended 
       * to this timecode_material. 
       * Adding extra timecode components causes a timecode redefinition 
       * during the file playin/playout.
       * For instance timecode_component("00:00:00:00", 50)  (with a frame rate set to 25)
       *              timecode_component("00:10:00:00", 100)
       * Appending these two timecode components will give the following timecode
       * playout: "00:00:00:00", "00:00:00:01", ..., "00:00:01:24", "00:00:10:00", etc. 
       * Returns true if successful
       */
      virtual bool CALLING_CONVENTION append_timecode_component(I_timecode_component* /* in */)=0;

      /*
       * Gets the timecode components of this material.
       * Returned timecode components must not be edited nor deleted.
       * Returns NULL if an error occured
       */
      virtual ordered_timecode_list* /* free */ CALLING_CONVENTION get_timecode_list()=0;

      /*
       * Frees the ordered_timecode_list returned by get_timecode_list
       * Returns true if successful
       */
      virtual bool CALLING_CONVENTION free_timecode_list(ordered_timecode_list* /* in */)=0;
    };

    /*
     * Retrieves a timecode material interface.
     * I_timecode_material** pointer to the newly allocated object
     * int64_t offset in edit units (usually 0)
     * rational edit rate of the subjacent timecode track.
     * duration is computed while appending timecode components.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewTimecodeMaterialInterface(I_timecode_material** /* out free */, 
                                                                               int64_t /* in */, 
                                                                               rational* /* in free */);

    /*
     * Frees a timecode material interface.
     * I_timecode_material** pointer to the object to be freed
     * User should not free a timecode material already appended to a material.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeTimecodeMaterialInterface(I_timecode_material** /* in out */);
  }
}

#endif
