//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the source clip, dm_source clip
//				and dm_segment interfaces.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#ifndef OC_I_TRACK_ITEM_HPP
#define OC_I_TRACK_ITEM_HPP

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_generic_material.hpp>
#include <mxf_tk/I_timecode.hpp>
#include <mxf_tk/I_umid.hpp>

#define INT64_ERROR   0x7fffffffffffffffLL
#define UINT64_ERROR  0xffffffffffffffffULL

namespace opencube
{
  namespace MXFTK_NAMESPACE
  {
    /**
     * A generic material track can be seen as a set of ordered "tiles" 
     * taken from tracks of concrete materials. track_item is the 
     * class describing one of these track tiles. It has three possible
     * derivations depending on the track type. These items are used to
     * specify an edition of the video/sound and metadata.
     * Used in a generic material's track, they describe assemblies of
     * concrete material's track segments.
     */
    class I_track_item
    {
    protected:
      I_track_item();
      virtual ~I_track_item();
    };

    /**
     * A source_clip can be found only on essence tracks (picture, sound, data).
     * get_referenced_material refers to the material where essence data will be retrieved.
     * get_referenced_material_umid returns the umid of the referenced material. The I_umid pointer
     * must not be destroyed.
     * get_referenced_track refers to the the sub_material's where essence data will be retrieved.
     * If the referenced material cannot be found then referenced material and referenced track 
     * will be set to NULL. This can be the case when reading OpAtom files where the referenced
     * material is stored in another OpAtom file.
     * get_start_position gives the edit units offset (relative to the origin of _sub_track, measured
     * in edit units of _sub_track) from where the essence must be read. It will return 0x7fffffffffffffffLL
     * if the referenced track is set to NULL.
     * get_end_position gives the last edit units offset (relative to the origin of _sub_track, measured
     * in edit units of _sub_track) from where the essence must be read. 0x7fffffffffffffff if duration is unknown.
     * get_start_timecode and get_end_timecode returns the tc_in and tc_out of this source_clip in this track.
     * get_duration gives the duration (in edit units from the track containing this source_clip) to be read on _sub_track. 
     * If returned value is -1, the duration is not known yet.
     * CAUTION !! If the track containing this source clip has an edit rate different from the edit rate of the 
     * referenced track; then start_position + duration != end_position.
     * _sub_track from _start_position. If returned value is -1, the duration is not known yet.
     */
    class I_source_clip : public I_track_item
    {
    public:
      virtual I_generic_material* CALLING_CONVENTION get_referenced_material() const=0;
      virtual const I_umid* CALLING_CONVENTION get_referenced_material_umid() const=0;
      virtual I_track* CALLING_CONVENTION get_referenced_track() const=0;
      virtual int64_t CALLING_CONVENTION get_start_position() const=0;
      virtual int64_t CALLING_CONVENTION get_end_position() const=0;
      virtual I_timecode* /* free */ CALLING_CONVENTION get_start_timecode() const=0;
      virtual I_timecode* /* free */ CALLING_CONVENTION get_end_timecode() const=0;
      virtual int64_t CALLING_CONVENTION get_duration() const=0;

      /*
       * Write into a buffer the essence stream of this source clip.
       * Should be called only while reading an mxf file.
       * uint8_t* buffer to be filled by the essence (allocated by MXF_TK user)
       * uint64_t number of bytes to be written (buffer size)
       * Returns the number of bytes effectively written.
       */
      virtual size_t CALLING_CONVENTION read_eu(uint8_t*, size_t)=0;

      /*
       * Returns the size of the current edit unit from this source clip
       * Returns 0xffffffffffffffff if an error occured.
       */
      virtual uint64_t CALLING_CONVENTION current_eu_size()=0;

      /*
       * Returns the current edit unit from this source clip (edit unit from 
       * the referenced track)
       * Returns 0x7fffffffffffffff if an error occured.
       */
      virtual int64_t CALLING_CONVENTION get_current_eu()=0;

      /*
       * Seek a given edit unit from this source clip (should be comprised
       * between start and end position). Returns true if successful.
       */
      virtual bool CALLING_CONVENTION seek_eu(int64_t)=0;

      /*
       * Seek next edit unit from this source_clip. Returns true if succcessful.
       */
      virtual bool CALLING_CONVENTION seek_next_eu()=0;

      /*
       * Seek the previous edit unit from this source clip. Returns true if succcessful.
       */
      virtual bool CALLING_CONVENTION seek_previous_eu()=0;

      /*
       * Returns the current timecode corresponding to the current edit unit
       * Timecode is measured along the track containing this source clip. 
       * Its value is always comprised between get_start_timecode and get_end_timecode
       * Returns NULL if an error occured.
       * Returned pointer must be destroyed by MxfTk user.
       */
      virtual I_timecode* /* free */ CALLING_CONVENTION get_current_timecode()=0;
    };

    /**
     * A dm_source_clip can be found only on descriptive metadata tracks.
     * get_referenced_material refers to the material where metadata will be retrieved.
     * get_referenced_material_umid returns the umid of the referenced material. The I_umid pointer
     * must not be destroyed;
     * get_referenced_track refers to the the sub_material's track  where metadata will be retrieved.
     * If the referenced material cannot be found then referenced material and referenced track 
     * will be set to NULL. This can be the case when reading OpAtom files where the referenced
     * material is stored in another OpAtom file.
     * get_start_position gives the edit units offset (relative to the origin of _sub_track, measured
     * in edit units of _sub_track) from where the essence must be read.
     * get_end_position gives the last edit units offset (relative to the origin of _sub_track, measured
     * in edit units of _sub_track) from where the essence must be read. 0x7fffffffffffffff if duration is unknown.
     * get_duration gives the duration (in edit units from the track containing this source_clip) to be read on _sub_track. 
     * If returned value is -1, the duration is not known yet.
     * CAUTION !! If the track containing this source clip has an edit rate different from the edit rate of the 
     * referenced track; then start_position + duration != end_position.
     * get_documented_tracks is a reference to the essence tracks that the metadata 
     * of this dm_source_clip is documenting. 
     */
    class I_dm_source_clip : public I_track_item
    {
    public:
      virtual I_generic_material* CALLING_CONVENTION get_referenced_material() const=0;
      virtual const I_umid* CALLING_CONVENTION get_referenced_material_umid() const=0;
      virtual I_track* CALLING_CONVENTION get_referenced_track() const=0;
      virtual int64_t CALLING_CONVENTION get_start_position() const=0;
      virtual int64_t CALLING_CONVENTION get_end_position() const=0;
      virtual I_timecode* /* free */ CALLING_CONVENTION get_start_timecode() const=0;
      virtual I_timecode* /* free */ CALLING_CONVENTION get_end_timecode() const=0;
      virtual int64_t CALLING_CONVENTION get_duration() const=0;
      virtual track_list* CALLING_CONVENTION get_documented_tracks() const=0;
    };

    class I_metadata_material;

    /**
     * A dm_segment can be found only on descriptive metadata tracks. It 
     * references a descriptive metadata material.
     * _sub_material refers to the material where metadata will be retrieved;
     * Information on duration and position on the track will be found in this material.
     * _documented_tracks is a reference to the essence tracks that the metadata 
     * of this dm_segment is documenting. 
     * Also look at the class I_metadata_material for a complete overview of
     * the metadata track features.
     */
    class I_dm_segment : public I_track_item
    {
    public:
      virtual I_metadata_material* CALLING_CONVENTION get_metadata_material() const=0;
      virtual track_list* CALLING_CONVENTION get_documented_tracks() const=0;
      virtual I_timecode* /* free */ CALLING_CONVENTION get_start_timecode() const=0;
      virtual I_timecode* /* free */ CALLING_CONVENTION get_end_timecode() const=0;
      virtual int64_t CALLING_CONVENTION get_duration() const=0;
    };

    /**
     * Use these functions to cast a track_item pointer into the appropriate type.
     * Do not use dynamic_cast on a track_item pointer in your code!
     * Failure to do so may result in crash.
     */
    extern "C" mxf_tk_API I_source_clip* CALLING_CONVENTION source_clip_cast(I_track_item*);
    extern "C" mxf_tk_API I_dm_source_clip* CALLING_CONVENTION dm_source_clip_cast(I_track_item*);
    extern "C" mxf_tk_API I_dm_segment* CALLING_CONVENTION dm_segment_cast(I_track_item*);
  }
}

#endif
