//------------------------------------------------------------------------------
//  Author:    	Thierry FRANZETTI
//  Comment:	MXF_TK: This example illustrates op1a mxf file creation.
//              The resulting file will be crypted.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#if defined(BC6)	// Borland C Builder 6.0
#include <vcl.h>
#endif

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_concrete_material.hpp>
#include <mxf_tk/I_op1a_file.hpp>

#include <iostream>
#include <list>

using namespace opencube::MXFTK_NAMESPACE;
using namespace std;

// MxfTk error handler
I_mxf_error_handler* err_h = NULL;

//Throws an exception when an error occured 
void throw_on_error(); 

//Called if an error occured.
void display_errors();

int main(int argc, char* argv[])
{
  if (argc < 4 )
  {
    cout << "usage : out.mxf out.xml source file [...source file]" << endl;
    exit(0);
  }

  // the op1a mxf file we are going to write.
  I_op1a_file* pi_op1a_file = NULL;
  // the interface to manipulate the source to be wrapped.
  I_concrete_material* pi_conc_mat = NULL;
  // this boolean is set to true if pi_conc_mat needs to be destroyed
  bool delete_conc_mat = false;
  // Path to the source files to be wrapped
  crypted_locators* pi_loc = NULL;

  try
  {
    wcout << L"Retrieve MxfTk Error Handler" << endl;
    if (!GetMxfErrorHandlerInterface(&err_h))
      throw 1;

    wcout << L"MxfTk Initialization" << endl;
    InitMXF_TK();
    throw_on_error();

    wcout << L"Create the op1a mxf file interface" << endl;
    NewOp1aFileInterface(&pi_op1a_file, argv[1]);
    throw_on_error();

    
    // Use this to tell MxfTk to generate index tables for this file
    wcout << L"Activate Index Tables" << endl;
    pi_op1a_file->set_parameter(INDEX_TABLE, "True");
    

    wcout << L"Build source files list" << endl;
    GetEmptyCryptedLocatorInterface(&pi_loc);
    throw_on_error();

    // elem are stored to be deleted after
    std::list<crypted_locator_element *> elem_list;

    for (int i=3; i<argc; i++)
    {
      crypted_locator_element * elem = NULL;
      uint64_t plaintext_off(0);

      uint8_t* cryptographic_key_id = new uint8_t[16];
      uint8_t* cipher_key = new uint8_t[16];

      for(unsigned int ii = 0; ii < 16; ii++)
      {
	cryptographic_key_id[ii] = ii;
	cipher_key[ii] = 16 - ii;
      }

      NewCryptedLocatorElementInterface(&elem, argv[i], plaintext_off, 
					cryptographic_key_id, cipher_key);
      
      pi_loc->add(&elem);
      elem_list.push_back(elem);

      delete[] cryptographic_key_id;
      delete[] cipher_key;
    }
    

    wcout << L"Create source material" << endl;
    NewCryptedMaterialInterface(&pi_conc_mat, pi_loc);
    throw_on_error();
    delete_conc_mat = true;

    for(std::list<crypted_locator_element *>::iterator it = elem_list.begin();
	it != elem_list.end(); it++)
    {
      FreeCryptedLocatorElementInterface(&(*it));
    }

    wcout << L"Attach material to the mxf file" << endl;
    pi_op1a_file->set_material(pi_conc_mat);
    throw_on_error();
    delete_conc_mat = false;  
    // delete_conc_mat set to false now coz pi_conc_mat will be destroyed 
    // upon deletion of pi_op1a_file (FreeOp1aFileInterface).

    wcout << L"Write mxf file on disk" << endl;
    pi_op1a_file->flush();
    throw_on_error();

    wcout << L"Wait termination" << endl;
    pi_op1a_file->wait_end_flush_thread();
    throw_on_error();

    wcout << L"Output header metadata XML file" << endl; // optional!
    pi_op1a_file->output_xml(argv[2]);
    throw_on_error();
  }
  catch (int)
  {
    wcout << "An error occured!" << std::endl;
    // Retrieving details of the error(s) and display it(them).
    display_errors();
  }

  wcout << L"Close Process" << endl;
  if (pi_op1a_file)
    FreeOp1aFileInterface(&pi_op1a_file);
  if (pi_conc_mat && delete_conc_mat)
    FreeConcreteMaterialInterface(&pi_conc_mat);
  if (pi_loc)
    FreeCryptedLocatorInterface(&pi_loc);

  return 0;
}

void throw_on_error()
{
  if (err_h->has_an_error())
    throw 1;
}

void display_errors()
{
  // Retrieve error handler
  if (!err_h)
    return;

  // Retrieve list of errors
  error_list* err_l = err_h->get_errors();
  if (err_l)
  {
    err_l->begin();
    I_mxf_error** ppi_mxf_error = NULL;
    while ((ppi_mxf_error = err_l->next()) != NULL) //go through the list
    {
      I_mxf_error* pi_error = *ppi_mxf_error;
      // and display each error 
      wcout << L"Error " << (int)pi_error->get_id() << L": " << pi_error->get_error_msg() << endl;
    }
    // free error list interface
    err_h->free_errors(err_l);
  }
}
