//------------------------------------------------------------------------------
//  Author:    	Thierry FRANZETTI
//  Comment:	MXF_TK: This example illustrates mxf file creation while streaming
//				the input essences in passive mode.
//              This is configured for creating XDCAM IMX files.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#if defined(BC6)	// Borland C Builder 6.0
#include <vcl.h>
#endif

#include <mxf_tk/I_concrete_material.hpp>
#include <mxf_tk/I_xdcam_dv_file.hpp>

#include <iostream>
#include <sstream>

#ifdef WIN32
#include <windows.h>
#endif

// Size of the input stream buffer
#define BUF_SIZE 16384

using namespace opencube::MXFTK_NAMESPACE;
using namespace std;

// MxfTk error handler
I_mxf_error_handler* err_h = NULL;

//Throws an exception when an error occured 
void throw_on_error(); 

//Called if an error occured.
void display_errors();

/* Derivation of the pure vitual class from MxfKt
Note that the function data_request was not reimplemented.
See passive mode example for a demonstration of use of the function data_request.
Here we use an implementation using files for conveniency
however the same mechanism could be used with data originating
from a true streaming device (linear storage device, Firewire port) 
*/
class I_user_stream_task : public I_essence_stream_task
{
public:
  // Constructor -> init source streams
  I_user_stream_task(unsigned int, char* argv[]);

  // Free source streams
  virtual ~I_user_stream_task();

  // Sets passive mode
  virtual bool CALLING_CONVENTION get_passive_mode() { return true; }

  // Sets the file information
  virtual void CALLING_CONVENTION get_stream_info(unsigned int stream_id, 
    essence_source& s, 
    bool& is_video, 
    unsigned int& nb_mpeg_es);

  // Sets the MPEG PS/TS substream info
  virtual void CALLING_CONVENTION get_mpeg_es_info(unsigned int stream_id, 
						   unsigned int sub_mpeg_es_id, 
						   essence_source& s,
						   bool& is_video);

  // Always called in active mode ! -> you must implement it!
  virtual size_t CALLING_CONVENTION data_request(unsigned int, uint8_t*, size_t);

  // This function will be called once by MxfTk to know how many source streams
  // are to be wrapped (for instance audio and video may originate from a different
  // Firewire port).
  virtual unsigned int CALLING_CONVENTION get_total_number_of_streams() { return _nb_streams; };

  // This function will be called once by MxfTk.
  // Sets the size of MxfTk internal buffer to bufferize input streams.
  // Here it is set to the input buffer size. Increasing its size may 
  // help to increase performances.
  // Set buffer size to 1Mb.
  virtual size_t CALLING_CONVENTION get_buffer_size(unsigned int) { return BUF_SIZE; };

protected:
  // File handle (simulating true stream)
  FILE ** files;
  // Total number of essences (streams) to be wrapped.
  unsigned int _nb_streams;
};

// Initalize nb_streams streaming devices (files here) 
I_user_stream_task::I_user_stream_task(unsigned int nb_streams, char* argv[])
: I_essence_stream_task()
{
  _nb_streams = nb_streams;
  // Open streaming devices.
  files = new FILE*[nb_streams];
  for (unsigned int i=0; i<nb_streams; i++)
  {
    files[i] = fopen(argv[i+2], "rb");
    if (ferror(files[i]))
      wcout << L"An error occured while opening the file " << argv[i+3] << endl;
  }
}

// Free streaming devices.
I_user_stream_task::~I_user_stream_task()
{
  for (unsigned int i=0; i<_nb_streams; i++)
  {
    fclose(files[i]);
  }
  delete [] files;
}

size_t CALLING_CONVENTION I_user_stream_task::data_request(unsigned int stream_id, uint8_t* buffer, size_t s_req)
{
  try
  {
    // If the streaming device is not ready the current process can be put asleep
    // with a mutex and woke up only once some data is ready.

    // Retrieve data from streaming device buffer
    size_t s_read = fread(buffer, sizeof(uint8_t), s_req, files[stream_id]);
    if (ferror(files[stream_id])) throw 1;

    return s_read;
  }
  catch(int)
  {
    wcout << L"An error occured!" << endl;
    // Retrieving details of the error(s) and display it(them).
    display_errors();
  }
  return 0;
}

void CALLING_CONVENTION I_user_stream_task::get_stream_info(unsigned int stream_id, 
                                                            essence_source& s, 
                                                            bool& is_video,
                                                            unsigned int& nb_mpeg_es)
{ 
  if (stream_id == 0)
  {
    s = ess_dv_iec;
    is_video = true;
    nb_mpeg_es = 0;
  }
  else
  {
    s = ess_aes3;
    is_video = false;
    nb_mpeg_es = 0;
  }
}

void CALLING_CONVENTION I_user_stream_task::get_mpeg_es_info(unsigned int/* stream_id*/, 
							     unsigned int sub_mpeg_es_id, 
							     essence_source&/* s*/, 
							     bool& is_video)
{
  if (sub_mpeg_es_id == 0)
  {
    is_video = true;
  }
  else
  {
    is_video = false;
  }
}

//*************************************************************************************************************
//*************************************************************************************************************
// main loop

int main(int argc, char* argv[])
{
  if (argc != 7 )
  {
    cout << "usage : out.mxf dv aes1 aes2 aes3 aes4" << endl;
    exit(0);
  }

  I_xdcam_dv_file* pi_xdcam_file = NULL;
  I_concrete_material* pi_conc_mat = NULL;

  try
  {
    wcout << L"Retrieve MxfTk Error Handler" << endl;
    if (!GetMxfErrorHandlerInterface(&err_h))
      throw 1;

    wcout << L"MxfTk Initialization" << endl;
    InitMXF_TK();
    throw_on_error();

    cout << "Create the Xdcam IMX file" << endl;
    NewXdcamDvFileInterface(&pi_xdcam_file, argv[1]);
    throw_on_error();

    wcout << L"Create input streaming device" << endl;
    // argc-2 is the number of input streams.
    I_user_stream_task s_task(argc-2, argv);

    wcout << L"Create source material" << endl;
    // Frame wrapping is always recommended to create streamable files.
    NewStreamMaterialInterface(&pi_conc_mat, &s_task, xdcam_wrapping);
    throw_on_error();

    wcout << L"Attach material to the file" << endl;
    pi_xdcam_file->set_material(pi_conc_mat); 
    throw_on_error();

    wcout << L"Launch the flush thread" << endl;
    pi_xdcam_file->flush(); 
    throw_on_error();
    // wait for complete stop of the flush thread (you MUST call
    // this function to deallocate the memory allocated by the flush).
    pi_xdcam_file->wait_end_flush_thread();
    wcout << L"Close Process" << endl;
  }
  catch(int)
  {
    wcout << L"An error occured!" << endl;
    // Retrieving details of the error(s) and display it(them).
    display_errors();
  }

  if (pi_xdcam_file)
    FreeXdcamDvFileInterface(&pi_xdcam_file);

  return 0;
}

void throw_on_error()
{
  if (err_h->has_an_error())
    throw 1;
}

void display_errors()
{
  // Retrieve error handler
  if (!err_h)
    return;

  // Retrieve list of errors
  error_list* err_l = err_h->get_errors();
  if (err_l)
  {
    err_l->begin();
    I_mxf_error** ppi_mxf_error = NULL;
    while ((ppi_mxf_error = err_l->next()) != NULL) //go through the list
    {
      I_mxf_error* pi_error = *ppi_mxf_error;
      // and display each error 
      wcout << L"Error " << (int)pi_error->get_id() << L": " << pi_error->get_error_msg() << endl;
    }
    // free error list interface
    err_h->free_errors(err_l);
  }
}
